from generics import *

from definitions import *

from GenericModel import * 

from GenericEvent import *

from GenericNetworkLayer import *

from GenericLinkLayer import *

from GenericApplicationLayer import *

from Experiment import Experiment

from OSIModel import AdHocNode




from enum import Enum

class EventTypes(Enum):
  INIT = "init"
  MFRB = "msgfrombottom"
  MFRT = "msgfromtop"
  MFRP = "msgfrompeer"

class ConnectorTypes(Enum):
  DOWN = "DOWN"
  UP = "UP"
  PEER = "PEER"

class MessageDestinationIdentifiers(Enum):
  LINKLAYERBROADCAST = -1,  # sinngle-hop broadcast, means all directly connected nodes
  NETWORKLAYERBROADCAST = -2  # For flooding over multiple-hops means all connected nodes to me over one or more links



import yaml as y

class TopoType:
  name = ""
  nodes = []

  def __init__(self, name, nodes, links):
      self.name = name
      self.nodes = nodes
      self.links = links

class ExperimentType: 
  name = ""
  topology_name = ""
  sampling_count = 0
  usrp_included = False

  def __init__(self, name, topology_name, sampling_count, include_hw):
    self.name = name
    self.topology_name = topology_name
    self.sampling_count = sampling_count
    self.usrp_included = include_hw
  
  

class AhcObject: 
  topologies = []
  experiments = []
  experiment_count = 0
  def __init__(self) -> None:
      pass

  def parse_data(self, inp): 
    data = y.load(inp)

    try:
      for topo in data.topologies:
        self.topologies.append(TopoType(topo.name, topo.nodes, topo.links))
      
      for exp in data.experiment:
        self.experiments.append(ExperimentType(exp.name, exp.topology, exp.sampling_count, exp.include_usrp))
    except AttributeError:
      print('Yaml file contains missing attributes')




from generics import *
from definitions import *
from enum import Enum
from threading import Thread, Lock
from random import sample
import networkx as nx
import itertools
import queue

inf = float('inf')
class Topology:
  nodes = {}
  channels = {}


  def construct_winslab_topology_with_channels(self, nodecount, nodetype, channeltype, context=None):
    
    self.construct_winslab_topology_without_channels(nodecount, nodetype, context)
    
    pairs = list(itertools.permutations(range(nodecount), 2))
    print("Pairs", pairs)
    self.G.add_edges_from(pairs)
    edges = list(self.G.edges)
    for k in edges:
      ch = channeltype(channeltype.__name__, str(k[0]) + "-" + str(k[1]))
      self.channels[k] = ch
      self.nodes[k[0]].connect_me_to_channel(ConnectorTypes.DOWN, ch)
      self.nodes[k[1]].connect_me_to_channel(ConnectorTypes.DOWN, ch)


  def construct_winslab_topology_without_channels(self, nodecount, nodetype, context=None):
    
    self.G = nx.Graph()
    self.G.add_nodes_from(range(nodecount))  # TODO : Change depending on the 

    nodes = list(self.G.nodes)
    for i in nodes:
      cc = nodetype(nodetype.__name__, i)
      self.nodes[i] = cc


  def construct_from_graph(self, G: nx.Graph, nodetype, channeltype, context=None):
    self.G = G
    nodes = list(G.nodes)
    edges = list(G.edges)
    self.compute_forwarding_table()
    for i in nodes:
      cc = nodetype(nodetype.__name__, i, self.ForwardingTable)
      self.nodes[i] = cc
    for k in edges:
      ch = channeltype(channeltype.__name__ + "-" + str(k[0]) + "-" + str(k[1]), str(k[0]) + "-" + str(k[1]))
      self.channels[k] = ch
      self.nodes[k[0]].connect_me_to_channel(ConnectorTypes.DOWN, ch)
      self.nodes[k[1]].connect_me_to_channel(ConnectorTypes.DOWN, ch)

  def construct_single_node(self, nodetype, instancenumber):
    self.singlenode = nodetype(nodetype.__name__, instancenumber)
    self.G = nx.Graph()
    self.G.add_nodes_from([0])
    self.nodes[0] = self.singlenode

  def construct_sender_receiver(self, sendertype, receivertype, channeltype):
    self.sender = sendertype(sendertype.__name__, 0)
    self.receiver = receivertype(receivertype.__name__, 1)
    ch = channeltype(channeltype.__name__, "0-1")
    self.G = nx.Graph()
    self.G.add_nodes_from([0, 1])
    self.G.add_edges_from([(0, 1)])
    self.nodes[self.sender.componentinstancenumber] = self.sender
    self.nodes[self.sender.componentinstancenumber] = self.receiver
    self.channels[ch.componentinstancenumber] = ch
    self.sender.connect_me_to_channel(ConnectorTypes.DOWN, ch)
    self.receiver.connect_me_to_channel(ConnectorTypes.DOWN, ch)

  def allpairs_shortest_path(self):
    return dict(nx.all_pairs_shortest_path(self.G))

  def shortest_path_to_all(self, myid):
    path = dict(nx.all_pairs_shortest_path(self.G))
    nodecnt = len(self.G.nodes)
    for i in range(nodecnt):
      print(path[myid][i])

  def start(self):
    N = len(self.G.nodes)
    self.compute_forwarding_table()
    # self.nodecolors = ['b'] * N
    # self.lock = Lock()

  def compute_forwarding_table(self):
    # N = len(self.G.nodes)
    # print(f"There are {N} nodes")
    # for i in range(N):
    #   for j in range(N):
    #     try:
    #       mypath = path[i][j]
    #       print(f"{i}to{j} path = {path[i][j]} nexthop = {path[i][j][1]}")
    #       self.ForwardingTable[i][j] = path[i][j][1]

    #       print(f"{i}to{j}path = NONE")
    #       self.ForwardingTable[i][j] = inf  # No paths
    #     except IndexError:
    #       print(f"{i}to{j} nexthop = NONE")
    #       self.ForwardingTable[i][j] = i  # There is a path but length = 1 (self)

    # all-seeing eye routing table contruction
    # def print_forwarding_table(self):
    #   registry.print_components()
    #   print('\n'.join([''.join(['{:4}'.format(item) for item in row])
    #                    for row in list(self.ForwardingTable.values())]))
    self.ForwardingTable = dict(nx.all_pairs_shortest_path(self.G))

  # returns the all-seeing eye routing based next hop id
  def get_next_hop(self, fromId, toId):
    try:
      retval = self.ForwardingTable[fromId][toId]
      return retval[1]
    except KeyError:
      return inf
    except IndexError:
      return fromId

  # Returns the list of neighbors of a node
  

  def get_predecessors(self, nodeId):
    return sorted([neighbor for neighbor in self.G.predecessors(nodeId)])

  def get_successors(self, nodeId):
    return sorted([neighbor for neighbor in self.G.neighbors(nodeId)])

  # Returns the list of neighbors of a node
  def get_neighbor_count(self, nodeId):
    # return len([neighbor for neighbor in self.G.neighbors(nodeId)])
    return self.G.degree[nodeId]

  def plot(self):
    # self.lock.acquire()
    # nx.draw(self.G, self.nodepos, node_color=self.nodecolors, with_labels=True, font_weight='bold')
    # plt.draw()
    print(self.nodecolors)
    # self.lock.release()

  def get_random_node(self):
    return self.nodes[sample(self.G.nodes(), 1)[0]]




import datetime

class GenericMessagePayload:

  def __init__(self, messagepayload):
    self.messagepayload = messagepayload

class GenericMessage:

  def __init__(self, header, payload):
    self.header = header
    self.payload = payload
    self.uniqueid = str(header.messagefrom) + "-" + str(header.sequencenumber)

class GenericMessageHeader:

  def __init__(self, messagetype, messagefrom, messageto, nexthop=float('inf'), interfaceid=float('inf'), sequencenumber=-1):
    self.messagetype = messagetype
    self.messagefrom = messagefrom
    self.messageto = messageto
    self.nexthop = nexthop
    self.interfaceid = interfaceid
    self.sequencenumber = sequencenumber

class Event:
  curr_event_id = 0

  def __init__(self, eventsource, event, eventcontent, fromchannel=None,
               eventid=-1):
    self.eventsource = eventsource
    self.event = event
    self.time = datetime.datetime.now()
    self.eventcontent = eventcontent
    self.fromchannel = fromchannel
    self.eventid = eventid
    if self.eventid == -1:
      self.eventid = self.curr_event_id
      self.curr_event_id += 1

  def __eq__(self, other) -> bool:
    if type(other) is not Event:
      return False

    return self.eventid == other.eventid

  def __hash__(self) -> int:
    return self.eventid



# from Channels import P2PFIFOPerfectChannel
from definitions import *
from generics import *
from GenericModel import *
from GenericApplicationLayer import *
from GenericLinkLayer import *
from GenericNetworkLayer import *
from GenericTransportLayer import *
from GenericChannel import P2PFIFOPerfectChannel, GenericChannel

class AHCChannelError(Exception):
  pass

class AdHocNode(GenericModel):

  def on_init(self, eventobj: Event):
    print(f"Initializing {self.componentname}.{self.componentinstancenumber}")

  def on_message_from_top(self, eventobj: Event):
    self.send_down(Event(self, EventTypes.MFRT, eventobj.eventcontent))

  def on_message_from_bottom(self, eventobj: Event):
    self.send_up(Event(self, EventTypes.MFRB, eventobj.eventcontent))

  def __init__(self, componentname, componentid, fw_table):
    super().__init__(componentname, componentid)

    self.appllayer = GenericApplicationLayer("ApplicationLayer", self.componentinstancenumber)
    self.netlayer = GenericNetworkLayer("NetworkLayer", self.componentinstancenumber, fw_table)      
    self.linklayer = GenericLinkLayer("LinkLayer", self.componentinstancenumber) 
    self.transportlayer = GenericTransportLayer("TransportLayer", self.componentinstancenumber) 

    self.appllayer.connect_me_to_component(ConnectorTypes.DOWN, self.transportlayer)
    self.transportlayer.connect_me_to_component(ConnectorTypes.DOWN, self.netlayer)
    self.netlayer.connect_me_to_component(ConnectorTypes.DOWN, self.linklayer)
    self.linklayer.connect_me_to_component(ConnectorTypes.DOWN, self)

    self.linklayer.connect_me_to_component(ConnectorTypes.UP, self.netlayer)
    self.netlayer.connect_me_to_component(ConnectorTypes.UP, self.transportlayer)
    self.transportlayer.connect_me_to_component(ConnectorTypes.UP, self.appllayer)
    self.connect_me_to_component(ConnectorTypes.UP, self.linklayer)
    
  
  def connect_to_layer(self, down, up, newLayer):
    newLayer.connect_me_to_component(ConnectorTypes.DOWN, down)
    newLayer.connect_me_to_component(ConnectorTypes.UP, up)
    down.connect_me_to_component(ConnectorTypes.UP, newLayer)
    up.connect_me_to_component(ConnectorTypes.DOWN, newLayer)

  def connect_me_to_channel(self, name, channel: GenericChannel):
    try:
        self.connectors[name] = channel
    except AttributeError:
        # self.connectors = ConnectorList()
        self.connectors[name] = channel
    connectornameforchannel = self.componentname + str(self.componentinstancenumber)
      
    channel.connect_me_to_component(connectornameforchannel, self)

  def replace_component(self, new:GenericModel, indx, args):
    match indx:
      case 0: # Physical Layer
        
        self.physicallayer:GenericModel = new(args) 
        self.physicallayer.connect_me_to_component(ConnectorTypes.UP, self.linklayer)
        self.linklayer.connect_me_to_component(ConnectorTypes.DOWN, self.physicallayer)
      case 1: # Link Layer
        self.linklayer = new(args)
        self.linklayer.connect_me_to_component(ConnectorTypes.UP, self.netlayer)
        self.netlayer.connect_me_to_component(ConnectorTypes.DOWN, self.linklayer)
        if self.physicallayer:
          self.physicallayer.connect_me_to_component(ConnectorTypes.UP, self.linklayer)
          self.linklayer.connect_me_to_component(ConnectorTypes.DOWN, self.physicallayer)
      case 2: # Network Layer
        self.netlayer = new(args)
        self.netlayer.connect_me_to_component(ConnectorTypes.UP, self.transportlayer)
        self.netlayer.connect_me_to_component(ConnectorTypes.DOWN, self.linklayer)
        self.linklayer.connect_me_to_channel(ConnectorTypes.UP, self.netlayer)
        self.transportlayer.connect_me_to_component(ConnectorTypes.DOWN, self.netlayer)
      case 3: # Transport Layer
        self.transportlayer = new(args)
        self.transportlayer.connect_me_to_component(ConnectorTypes.UP, self.appllayer)
        self.transportlayer.connect_me_to_component(ConnectorTypes.DOWN, self.netlayer)
        self.netlayer.connect_me_to_channel(ConnectorTypes.UP, self.transportlayer)
        self.appllayer.connect_me_to_component(ConnectorTypes.DOWN, self.transportlayer)
      case 4: # Application Layer
        self.appllayer = new(args)
        self.transportlayer.connect_me_to_channel(ConnectorTypes.UP, self.appllayer)
        self.appllayer.connect_me_to_component(ConnectorTypes.DOWN, self.transportlayer)





import random
import time
from enum import Enum

from definitions import MessageDestinationIdentifiers
from generics import *
from GenericEvent import *
from GenericModel import GenericModel

class BroadcastingEventTypes(Enum):
  BROADCAST = "broadcast"

# define your own message types
class BroadcastingMessageTypes(Enum):
  SIMPLEFLOOD = "SIMPLEFLOOD"

# define your own message header structure
class BroadcastingMessageHeader(GenericMessageHeader):
  pass

# define your own message payload structure
class BroadcastingMessagePayload(GenericMessagePayload):
  pass

class ControlledFlooding(GenericModel):
  def on_init(self, eventobj: Event):
    self.uniquebroadcastidentifier = 1
    self.broadcastdb = []
    if self.componentinstancenumber == 0  :
      self.send_self(Event(self, EventTypes.MFRT, None))

  def senddownbroadcast(self, eventobj: Event, whosends, sequencenumber):
    applmsg = eventobj.eventcontent
    destination = MessageDestinationIdentifiers.NETWORKLAYERBROADCAST
    nexthop = MessageDestinationIdentifiers.LINKLAYERBROADCAST
    print(f"{self.componentinstancenumber} will SEND a message to {destination} over {nexthop}")
    hdr = BroadcastingMessageHeader(BroadcastingMessageTypes.SIMPLEFLOOD, whosends, destination,
                                    nexthop, sequencenumber)
    payload = applmsg
    broadcastmessage = GenericMessage(hdr, payload)
    self.send_down(Event(self, EventTypes.MFRT, broadcastmessage))
    self.broadcastdb.append(broadcastmessage.uniqueid)

  def update_topology(self):
    Topology().nodecolors[self.componentinstancenumber] = 'r'
    Topology().plot()

  def on_broadcast(self, eventobj: Event):
    self.update_topology()
    self.uniquebroadcastidentifier = self.uniquebroadcastidentifier + 1
    self.senddownbroadcast(eventobj, self.componentinstancenumber, self.uniquebroadcastidentifier)
        
  def on_message_from_top(self, eventobj: Event):
    self.update_topology()
    evt = Event(self, BroadcastingEventTypes.BROADCAST, eventobj.eventcontent)
    self.send_self(evt)

  def on_message_from_bottom(self, eventobj: Event):
    msg = eventobj.eventcontent
    hdr = msg.header
    payload = msg.payload
    if hdr.messagetype == BroadcastingMessageTypes.SIMPLEFLOOD:
      if hdr.messageto == self.componentinstancenumber or hdr.messageto == MessageDestinationIdentifiers.NETWORKLAYERBROADCAST:  # Add if broadcast....
        if msg.uniqueid in self.broadcastdb:
          pass  # we have already handled this flooded message
        else:
          # Send to higher layers
          self.update_topology()
          self.send_up(Event(self, EventTypes.MFRB, payload))
          # Also continue flooding once
          time.sleep(random.randint(1, 3))
          self.senddownbroadcast(eventobj, eventobj.eventcontent.header.messagefrom,
                                 eventobj.eventcontent.header.sequencenumber)

  def __init__(self, componentname, componentinstancenumber):
    super().__init__(componentname, componentinstancenumber)
    self.eventhandlers[BroadcastingEventTypes.BROADCAST] = self.on_broadcast
    # add events here


from cgi import print_exception
from cmath import e
import queue
from typing import Dict
from typing import ClassVar, Generic
from generics import *
from definitions import *
from Topology import *
from threading import Thread, Lock
from timeit import default_timer as timer
from random import sample


class GenericModel:

    def __init__(self, componentname, componentinstancenumber, context=None, configurationparameters=None, num_worker_threads=1):
        self.context = context
        self.configurationparameters = configurationparameters
        self.eventhandlers = {EventTypes.INIT: self.on_init, EventTypes.MFRB: self.on_message_from_bottom,
                            EventTypes.MFRT: self.on_message_from_top, EventTypes.MFRP: self.on_message_from_peer}
        # Add default handlers to all instantiated components.
        # If a component overwrites the __init__ method it has to call the super().__init__ method
        self.inputqueue = queue.Queue()
        self.componentname = componentname
        self.componentinstancenumber = componentinstancenumber
        self.num_worker_threads = num_worker_threads
        self.connectors = {}
        self.terminated = False

        for i in range(self.num_worker_threads):
            t = Thread(target=self.queue_handler, args=[self.inputqueue])
            t.daemon = True
            t.start()

        try:
            if self.connectors:
                pass
        except AttributeError:
            self.connectors = {}
            # self.connectors = ConnectorList()

        #TODO: Handle This Part 
        # for i in range(self.num_worker_threads):
        #     t = Thread(target=self.queue_handler, args=[self.inputqueue])
        #     t.daemon = True
        #     t.start()

        # self.registry = ComponentRegistry()
        # self.registry.add_component(self)

   

    def send_down(self, event: Event):
        try:
            self.connectors[ConnectorTypes.DOWN].on_message_from_top(event)
            
        except Exception as e:
            raise(f"Cannot send message to Down Connector {self.componentname } -- {self.componentinstancenumber}")
            

    def send_up(self, event: Event):
        try:
            self.connectors[ConnectorTypes.UP].eventhandlers[EventTypes.MFRB](event)
            # for p in self.connectors[ConnectorTypes.UP]:
            #     p.trigger_event(event)
        except:
            pass

    def send_peer(self, event: Event):
        try:
            for p in self.connectors[ConnectorTypes.PEER]:
                p.trigger_event(event)
        except:
            pass

    def connect_me_to_component(self, name, component):
        self.connectors[name] = component
    
    def on_message_from_bottom(self, eventobj: Event):
        print(f"{EventTypes.MFRB} {self.componentname}.{self.componentinstancenumber}")

    def on_message_from_top(self, eventobj: Event):
        print(f"{EventTypes.MFRT}  {self.componentname}.{self.componentinstancenumber}")

    def on_message_from_peer(self, eventobj: Event):
        print(f"{EventTypes.MFRP}  {self.componentname}.{self.componentinstancenumber}")


    def on_init(self, eventobj: Event):
        if self.componentinstancenumber == 0:
            message_header = GenericMessageHeader("INITIATE  -> ", +str(self.componentname),
                                                  "Component-" + str(self.componentinstancenumber))
            message = GenericMessage(message_header, "")
            kickstarter = Event(self, EventTypes.MFRT, message)
            self.send_down(kickstarter)
            print(f"{self.componentname} - {self.componentinstancenumber} sends an INITIATE to Coordinator")
            self.start_time = timer()

    def queue_handler(self, myqueue):
        while not self.terminated:
            workitem = myqueue.get()
            if workitem.event in self.eventhandlers:
                self.on_pre_event(workitem)
                self.eventhandlers[workitem.event](eventobj=workitem)  # call the handler
            else:
                print(f"Event Handler: {workitem.event} is not implemented")
            myqueue.task_done()

    def connect_me_to_channel(self, name, channel):
        
        try:
            self.connectors[name] = channel
        except AttributeError:
            # self.connectors = ConnectorList()
            self.connectors[name] = channel

    def on_connected_to_channel(self, name, channel):
        print(f"Connected channel-{name} by component-{self.componentinstancenumber}:{channel.componentinstancenumber}")

    def trigger_event(self, eventobj: Event):
        self.inputqueue.put_nowait(eventobj)

    def on_pre_event(self, event):
        pass


from enum import Enum
from generics import *
from GenericModel import GenericModel
from definitions import *

class TransportLayerMessages(Enum):
  TRANSPORT_MSG = "TRANSPORT_MSG"

class GenericTransportLayer(GenericModel):

  def on_message_from_top(self, eventobj: Event):
    abovehdr = eventobj.eventcontent.header
    hdr = None
    if abovehdr.messageto == MessageDestinationIdentifiers.NETWORKLAYERBROADCAST:
      hdr = GenericMessageHeader(TransportLayerMessages.TRANSPORT_MSG, self.componentinstancenumber,
                                   MessageDestinationIdentifiers.LINKLAYERBROADCAST,nexthop=MessageDestinationIdentifiers.LINKLAYERBROADCAST)
    else:
      #if we do not broadcast, use nexthop to determine interfaceid and set hdr.interfaceid
      myinterfaceid = str(self.componentinstancenumber) + "-" + str(abovehdr.nexthop)
      hdr = GenericMessageHeader(TransportLayerMessages.TRANSPORT_MSG, self.componentinstancenumber,
                                   abovehdr.messageto, nexthop=abovehdr.nexthop, interfaceid=myinterfaceid)


    payload = eventobj.eventcontent
    msg = GenericMessage(hdr, payload)
    self.send_down(Event(self, EventTypes.MFRT, msg))

  def on_message_from_bottom(self, eventobj: Event):
    msg = eventobj.eventcontent
    hdr = msg.header
    payload = msg.payload
    if hdr.messageto == self.componentinstancenumber or hdr.messageto == MessageDestinationIdentifiers.LINKLAYERBROADCAST:
      self.send_up(Event(self, EventTypes.MFRB, payload,
                         eventobj.fromchannel))  # doing decapsulation by just sending the payload
    else:
      pass



import queue
from typing import ClassVar, Generic

from generics import *
from definitions import *
from Topology import *
from threading import Thread, Lock
from random import sample
from OSIModel import *
from generics import *

class GenericEvent:
  curr_event_id = 0

  def __init__(self, eventsource, event, eventcontent, fromchannel=None,
               eventid=-1):
    self.eventsource = eventsource
    self.event = event
    self.time = datetime.datetime.now()
    self.eventcontent = eventcontent
    self.fromchannel = fromchannel
    self.eventid = eventid
    if self.eventid == -1:
      self.eventid = self.curr_event_id
      self.curr_event_id += 1

  def __eq__(self, other) -> bool:
    if type(other) is not Event:
      return False

    return self.eventid == other.eventid

  def __hash__(self) -> int:
    return self.eventid


from setuptools import setup, find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
  # TODO: Fix the configs in this part
  name='adhoccomputing',
  version='2.0.0',    
  description="Ad Hoc Computing Library",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url='https://github.com/cengwins/ahc',
  author='CengWins',
  author_email='ahc@ceng.metu.edu.tr',
  license='BSD 2-clause',
  packages=['adhoccomputing'],
  install_requires=[
    'matplotlib',
    'mnist',
    'networkx',
    'numpy',
    'pandas',
    'Pillow',
    'pygraphviz',
    'scikit_learn',
    'seaborn',
    'setuptools',
    'torch',
    'yaml'
  ],
  project_urls={
        "Bug Tracker": "https://github.com/cengwins/ahc/issues",
        "Simulator Website": "https://ahc.ceng.metu.edu.tr"
  },
  classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
  python_requires=">=3.10",
)


from generics import *
from GenericModel import GenericModel
from definitions import *
import random
import time 

# define your own message types
class ApplicationLayerMessageTypes(Enum):
    PROPOSE = "PROPOSE"
    ACCEPT = "ACCEPT"


# define your own message header structure
class ApplicationLayerMessageHeader(GenericMessageHeader):
    pass


# define your own message payload structure
class ApplicationLayerMessagePayload(GenericMessagePayload):
    pass


class GenericApplicationLayer(GenericModel):

    def on_init(self, eventobj: Event):
        print(f"Initializing {self.componentname}.{self.componentinstancenumber}")

        if self.componentinstancenumber == 0:
            # destination = random.randint(len(Topology.G.nodes))
            destination = 1
            hdr = ApplicationLayerMessageHeader(ApplicationLayerMessageTypes.PROPOSE, self.componentinstancenumber,
                                                destination)
            payload = ApplicationLayerMessagePayload("23")
            proposalmessage = GenericMessage(hdr, payload)
            randdelay = random.randint(0, 5)
            time.sleep(randdelay)
            self.send_self(Event(self, "propose", proposalmessage))
        else:
            pass

    def on_message_from_bottom(self, eventobj: Event):
        try:
            applmessage = eventobj.eventcontent
            hdr = applmessage.header
            if hdr.messagetype == ApplicationLayerMessageTypes.ACCEPT:
                print(
                    f"Node-{self.componentinstancenumber} says Node-{hdr.messagefrom} has sent {hdr.messagetype} message")
            elif hdr.messagetype == ApplicationLayerMessageTypes.PROPOSE:
                print(
                    f"Node-{self.componentinstancenumber} says Node-{hdr.messagefrom} has sent {hdr.messagetype} message")
        except AttributeError:
            print("buradan geldi")
            print("Attribute Error")






import networkx as nx
import yaml as ym
from Parser import AhcObject

class Experiment: 
    AhcObj = AhcObject()

    def construct_from_nx_graph(self, graph: nx.Graph, node_type):
        pass 

    def construct_from_graph(self, graph, node_type):
        pass

    def construct_from_dict(self, dict: dict):
        pass

    def construct_from_yaml(self, yaml):
        AhcObject.parse_data(yaml)
        

    

